package StarFlight;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/** Am Betrachter hinter einer Textzeile vorbeiziehende Sterne.<BR>
    Ben&ouml;tigt Java in der Version 1.1 !!
  */
public final class StarFlight extends java.applet.Applet
                                implements PaintListener,
                                           MouseListener,
                                           MouseMotionListener
{
  // Groesze der Flaeche
  private int MaxX=0, MaxY=0;

  // Eigenschaften des auszugebenden Textes
  private String Textline;
  private int Textsize;
  private Color Textcolor;
  private Rectangle Text;

  // Die Anzahl der Sterne
  private int AnzStars;

  // Der Thread zum Weiterbewegen
  private StarRunner R = null;
  private int SleepTime;

  // das offscreen Image und sein Graphics-Kontext
  private Image OSimage;
  private Graphics img_G;


  // Auslesen der Parameter, Groesze festlegen, Bildpuffer anlegen
  public void init()
  {
    // Parameter empfangen...
    try
    {
      AnzStars = Integer.parseInt(getParameter("stars"));
    } catch (Exception e) {AnzStars = 100;};

    try
    {
      SleepTime = Integer.parseInt(getParameter("sleep"));
    } catch (Exception e) {SleepTime = 50;};

    try
    {
      Textline = getParameter("text");
    } catch (Exception e) {Textline = "";};

    try
    {
      Textsize = Integer.parseInt(getParameter("textsize"));
    } catch (Exception e) {Textsize = 8;};

    try
    {
      String ColorName = getParameter("textcolor");
      if      (ColorName.equals("black")) Textcolor=Color.black;
      else if (ColorName.equals("red"))   Textcolor=Color.red;
      else if (ColorName.equals("blue"))  Textcolor=Color.blue;
      else if (ColorName.equals("green")) Textcolor=Color.green;
      else Textcolor = Color.white;
    }
    catch (Exception e) {Textcolor = Color.white;}

    // nun alles Aufloesungsabhaengige erledigen

    // Groesse ggf. korrigieren
    if (getSize().width  < 60)
      resize(60,getSize().height);
    if (getSize().height < 25)
      resize(getSize().width,25);

    // Ausmasze merken
    MaxX = getSize().width;
    MaxY = getSize().height;

    // Offscreen-Image anlegen
    OSimage = createImage(MaxX,MaxY);

    // Graphics-Objekt fuer Offscreen-Image ermitteln
    img_G = OSimage.getGraphics();

    // und den Bereich loeschen
    img_G.setColor(Color.black);
    img_G.fillRect(0,0,MaxX,MaxY);

    // Ausgabe des Textes schon hier vorbereiten
    Font F = img_G.getFont();
    if (F.isBold()){
      img_G.setFont(new Font(F.getName(),F.getStyle(),Textsize));
    }
    else{
      img_G.setFont(new Font(F.getName(),F.getStyle()+F.BOLD,Textsize));
    }
    FontMetrics FM = img_G.getFontMetrics();

    Text = new Rectangle(0,0, FM.stringWidth(Textline),
						 FM.getMaxAscent() - FM.getMaxDescent());
    centerText();
  } // init()


  // zentrierte Ausrichtung des Textes
  private void centerText()
  {
    Text.x = (MaxX - Text.width ) / 2;
    Text.y = (MaxY + Text.height) / 2;
  } // centerText()


  public void destroy()
  {
	  if (OSimage != null)
	  {
		  OSimage.flush();
		  OSimage = null;
	  }
  }

  // Anhalten der Ausgabe (und Berechnung)
  public void stop()
  {
    this.removeMouseMotionListener(this);
    this.removeMouseListener(this);

    if ((R != null) && R.isAlive())
	{
      R.stopWork(); // Thread zum Ende bringen
      try {
        R.join();
      }
      catch (InterruptedException e){};
    }

    R = null;
  } // stop()


  // Beginn der Ausgabe (und Berechnung)
  public void start()
  {
    if ((R != null) && R.isAlive())
    {
      R.stopWork();
      try { R.join(); }
      catch (InterruptedException e){};
    } // if

    R = new StarRunner(AnzStars, MaxX,MaxY, this);
    R.setSleepTime(SleepTime);
    R.start();

    this.addMouseMotionListener(this);
    this.addMouseListener(this);
  } // start()


  public void update(Graphics g)
  {
    paint(g);
  } // update()


  public void paint(Graphics g)
  {
    // Offscreen-Image ausgeben
    g.drawImage(OSimage,0,0,null);
  } // paint()


  // --- Interface StarFlight.PaintListener --------------------
  public void paintingsPrepared()
  {
    // Offscreen Bitmap loeschen
    img_G.setColor(Color.black);
    img_G.fillRect(0,0,MaxX,MaxY);

    // Sterne zeichnen
    R.draw(img_G);

    // Text ueber dem Image ausgeben
    img_G.setColor(Textcolor);
    img_G.drawString(Textline, Text.x, Text.y);

    // und Applet neu zeichnen lassen...
    repaint();
  } // paintingsDone()


  // ---  Interface java.awt.event.MouseListener ------------------

  /** unbenutzt */
  public void mouseClicked(MouseEvent e)
  {} // mouseClicked()

  /** unbenutzt */
  public void mousePressed(MouseEvent parm1)
  {
  } // mousePressed()

  /** unbenutzt */
  public void mouseReleased(MouseEvent parm1)
  {} // mouseReleased()

  /** unbenutzt */
  public void mouseEntered(MouseEvent parm1)
  {} // mouseEntered()

  /** Beim Verlassen den Text wieder zentrieren. */
  public void mouseExited(MouseEvent parm1)
  {
    centerText();
  } // mouseExited()


  // ---  Interface java.awt.event.MouseMotionListener ------------

  /** unbenutzt */
  public void mouseDragged(MouseEvent parm1)
  {}

  /** Text von der Maus weg verschieben. */
  public synchronized void mouseMoved(MouseEvent e)
  {
    int mX = e.getX();
    int mY = e.getY();

    int x1 = Text.x;
    int x2 = x1 + Text.width;
    int y2 = Text.y;
    int y1 = y2 - Text.height;

    int dX = 0, dY = 0;

    // nur auszerhalb der Textgrenzen ueberpruefen !
    if (mX < x1)
      dX = x1 - mX;
    else if (mX > x2)
      dX = x2 - mX;

    // nur auszerhalb der Textgrenzen ueberpruefen !
    if (mY < y1)
      dY = y1 - mY;
    else if (mY > y2)
      dY = y2 - mY;

    // Abstand zum Text bestimmen
    double d = Math.sqrt( dX*dX + dY*dY );

    if (d != 0)
    {
      Text.x += (int) (5 * dX / d);
      Text.y += (int) (5 * dY / d);
    } // if

  } // mouseMoved()

} // class StarFlight

