package StarFlight;

import java.awt.Graphics;
import java.awt.Color;

// der Thread zum Weiterbewegen und Zeichnen der Sterne
final class StarRunner extends Thread
{
  // ein Array aus Sternen...
  private Star[] Stars;

  // minimale Zeit zwischen zwei Einzelbildern
  private long FrameDelay;

  // die 'Bildschirmgroesse' fuer das Sternentreiben
  private int MaxX,MaxY;

  // Nur ein PaintListener wird benachrichtigt...
  private PaintListener listener = null;


  // Das Flag ist true, solange der Thread weiterlaufen soll.
  private boolean goOn = true;


  // Constructor: Sternenarray initialisieren
  public StarRunner(int AnzStars, int MaxX, int MaxY, PaintListener pl)
  {
    listener = pl;
    setSize(MaxX,MaxY);
    Stars = new Star[AnzStars];
    for (int i=Stars.length; i>0; Stars[--i] = new Star(MaxX,MaxY) );
  } // StarRunner()



  public void setSleepTime(int delay)
  {
    FrameDelay = delay;
  }

  public void setSize(int Xmax, int Ymax)
  {
    MaxX = Xmax; MaxY = Ymax;
  }

  public void stopWork()
  {
    goOn = false;
  } // stopWork()



  // Zeichnen aller Sterne
  public void draw(Graphics G)
  {
    for (int i=Stars.length; i > 0; Stars[--i].drawMe(G));
  }

  // Loeschen aller Sterne
  public void undraw(Graphics G)
  {
    for (int i=Stars.length; i > 0; Stars[--i].undrawMe(G));
  }

  // schrittweises Weiterbewegen der Sterne
  public void run()
  {
    long Wartezeitende, Restzeit;

    Wartezeitende = System.currentTimeMillis();
    while (goOn){
      Wartezeitende += FrameDelay;

      Restzeit = Wartezeitende - System.currentTimeMillis();
      if (Restzeit > 0){

      	if (listener != null)
      	  listener.paintingsPrepared();

        try
        { sleep(Restzeit); }
        catch (InterruptedException e){}
      }

      for (int i=Stars.length; i>0; Stars[--i].moveOn(MaxX,MaxY));
    }
  } // run()

} // class StarRunner